# -*- coding: utf-8 -*-
# Copyright (c) 2019 Ansible Project
# Simplified BSD License (see licenses/simplified_bsd.txt or https://opensource.org/licenses/BSD-2-Clause)

from __future__ import annotations as _annotations

import datetime as _datetime
import typing as _t

from ansible.module_utils._internal import _traceback, _plugin_exec_context
from ansible.module_utils.common import messages as _messages
from ansible.module_utils import _internal

_UNSET = _t.cast(_t.Any, object())


def warn(warning: str) -> None:
    """Record a warning to be returned with the module result."""
    # DTFIX-RELEASE: shim to controller display warning like `deprecate`
    _global_warnings[_messages.WarningSummary(
        details=(
            _messages.Detail(msg=warning),
        ),
        formatted_traceback=_traceback.maybe_capture_traceback(_traceback.TracebackEvent.WARNING),
    )] = None


def deprecate(
    msg: str,
    version: str | None = None,
    date: str | _datetime.date | None = None,
    collection_name: str | None = _UNSET,
    *,
    help_text: str | None = None,
    obj: object | None = None,
) -> None:
    """
    Record a deprecation warning to be returned with the module result.
    The `obj` argument is only useful in a controller context; it is ignored for target-side callers.
    """
    if isinstance(date, _datetime.date):
        date = str(date)

    # deprecated: description='enable the deprecation message for collection_name' core_version='2.23'
    # if collection_name is not _UNSET:
    #     deprecate('The `collection_name` argument to `deprecate` is deprecated.', version='2.27')

    if _internal.is_controller:
        _display = _internal.import_controller_module('ansible.utils.display').Display()
        _display.deprecated(
            msg=msg,
            version=version,
            date=date,
            help_text=help_text,
            obj=obj,
        )

        return

    _global_deprecations[_messages.DeprecationSummary(
        details=(
            _messages.Detail(msg=msg, help_text=help_text),
        ),
        formatted_traceback=_traceback.maybe_capture_traceback(_traceback.TracebackEvent.DEPRECATED),
        version=version,
        date=date,
        plugin=_plugin_exec_context.PluginExecContext.get_current_plugin_info(),
    )] = None


def get_warning_messages() -> tuple[str, ...]:
    """Return a tuple of warning messages accumulated over this run."""
    # DTFIX-RELEASE: add future deprecation comment
    return tuple(item._format() for item in _global_warnings)


_DEPRECATION_MESSAGE_KEYS = frozenset({'msg', 'date', 'version', 'collection_name'})


def get_deprecation_messages() -> tuple[dict[str, _t.Any], ...]:
    """Return a tuple of deprecation warning messages accumulated over this run."""
    # DTFIX-RELEASE: add future deprecation comment
    return tuple({key: value for key, value in item._as_simple_dict().items() if key in _DEPRECATION_MESSAGE_KEYS} for item in _global_deprecations)


def get_warnings() -> list[_messages.WarningSummary]:
    """Return a list of warning messages accumulated over this run."""
    return list(_global_warnings)


def get_deprecations() -> list[_messages.DeprecationSummary]:
    """Return a list of deprecations accumulated over this run."""
    return list(_global_deprecations)


_global_warnings: dict[_messages.WarningSummary, object] = {}
"""Global, ordered, de-deplicated storage of acculumated warnings for the current module run."""

_global_deprecations: dict[_messages.DeprecationSummary, object] = {}
"""Global, ordered, de-deplicated storage of acculumated deprecations for the current module run."""



from __future__ import annotations

import typing as t

from ansible.module_utils.common.messages import SummaryBase, Detail

_TSummaryBase = t.TypeVar('_TSummaryBase', bound=SummaryBase)


def make_summary(summary_type: type[_TSummaryBase], *details: Detail, formatted_traceback: t.Optional[str] = None, **kwargs) -> _TSummaryBase:
    """Utility factory method to avoid inline tuples."""
    return summary_type(details=details, formatted_traceback=formatted_traceback, **kwargs)



from __future__ import annotations

from ansible.module_utils.datatag import deprecate_value
from ansible.plugins.action import ActionBase


class ActionModule(ActionBase):
    def run(self, tmp=None, task_vars=None):
        result = super().run(tmp, task_vars)
        result.update(deprecated_thing=deprecate_value("deprecated thing", msg="Deprecated thing is deprecated.", removal_version='999.999'))

        self._display.deprecated("did a deprecated thing", version="999.999")

        return result



"""
Message contract definitions for various target-side types.

These types and the wire format they implement are currently considered provisional and subject to change without notice.
A future release will remove the provisional status.
"""

from __future__ import annotations as _annotations

import sys as _sys
import dataclasses as _dataclasses

# deprecated: description='typing.Self exists in Python 3.11+' python_version='3.10'
from ..compat import typing as _t

from ansible.module_utils._internal import _datatag

if _sys.version_info >= (3, 10):
    # Using slots for reduced memory usage and improved performance.
    _dataclass_kwargs = dict(frozen=True, kw_only=True, slots=True)
else:
    # deprecated: description='always use dataclass slots and keyword-only args' python_version='3.9'
    _dataclass_kwargs = dict(frozen=True)


@_dataclasses.dataclass(**_dataclass_kwargs)
class PluginInfo(_datatag.AnsibleSerializableDataclass):
    """Information about a loaded plugin."""

    requested_name: str
    """The plugin name as requested, before resolving, which may be partially or fully qualified."""
    resolved_name: str
    """The resolved canonical plugin name; always fully-qualified for collection plugins."""
    type: str
    """The plugin type."""


@_dataclasses.dataclass(**_dataclass_kwargs)
class Detail(_datatag.AnsibleSerializableDataclass):
    """Message detail with optional source context and help text."""

    msg: str
    formatted_source_context: _t.Optional[str] = None
    help_text: _t.Optional[str] = None


@_dataclasses.dataclass(**_dataclass_kwargs)
class SummaryBase(_datatag.AnsibleSerializableDataclass):
    """Base class for an error/warning/deprecation summary with details (possibly derived from an exception __cause__ chain) and an optional traceback."""

    details: _t.Tuple[Detail, ...]
    formatted_traceback: _t.Optional[str] = None

    def _format(self) -> str:
        """Returns a string representation of the details."""
        # DTFIX-RELEASE: eliminate this function and use a common message squashing utility such as get_chained_message on instances of this type
        return ': '.join(detail.msg for detail in self.details)

    def _post_validate(self) -> None:
        if not self.details:
            raise ValueError(f'{type(self).__name__}.details cannot be empty')


@_dataclasses.dataclass(**_dataclass_kwargs)
class ErrorSummary(SummaryBase):
    """Error summary with details (possibly derived from an exception __cause__ chain) and an optional traceback."""


@_dataclasses.dataclass(**_dataclass_kwargs)
class WarningSummary(SummaryBase):
    """Warning summary with details (possibly derived from an exception __cause__ chain) and an optional traceback."""


@_dataclasses.dataclass(**_dataclass_kwargs)
class DeprecationSummary(WarningSummary):
    """Deprecation summary with details (possibly derived from an exception __cause__ chain) and an optional traceback."""

    version: _t.Optional[str] = None
    date: _t.Optional[str] = None
    plugin: _t.Optional[PluginInfo] = None

    @property
    def collection_name(self) -> _t.Optional[str]:
        if not self.plugin:
            return None

        parts = self.plugin.resolved_name.split('.')

        if len(parts) < 2:
            return None

        collection_name = '.'.join(parts[:2])

        # deprecated: description='enable the deprecation message for collection_name' core_version='2.23'
        # from ansible.module_utils.datatag import deprecate_value
        # collection_name = deprecate_value(collection_name, 'The `collection_name` property is deprecated.', removal_version='2.27')

        return collection_name

    def _as_simple_dict(self) -> _t.Dict[str, _t.Any]:
        """Returns a dictionary representation of the deprecation object in the format exposed to playbooks."""
        result = self._as_dict()
        result.update(
            msg=self._format(),
            collection_name=self.collection_name,
        )

        return result
